#include "Lin2db.h"


#if defined(_CRT_SECURE_NO_DEPRECATE) && !defined(_CRT_SECURE_NO_WARNINGS)
#define _CRT_SECURE_NO_WARNINGS
#endif

CLineageDatabase g_DB;
CRITICAL_SECTION g_DBCS;

#define MAX_BUFFER 256
#define MAX_COL_NAME_LEN 32

void CLineageDatabase::SQLAlloc()
{

	SQLAllocHandle(SQL_HANDLE_ENV, SQL_NULL_HANDLE, &m_hEnv);
	SQLSetEnvAttr(m_hEnv, SQL_ATTR_ODBC_VERSION, (void*)SQL_OV_ODBC3, 0); 
	SQLAllocHandle(SQL_HANDLE_DBC, m_hEnv, &m_hDbc); 
}

void CLineageDatabase::SQLFree()
{
	SQLFreeHandle(SQL_HANDLE_DBC, m_hDbc);
	SQLFreeHandle(SQL_HANDLE_ENV, m_hEnv);
	m_hDbc = NULL;
	m_hEnv = NULL;
}

void CLineageDatabase::Close()
{
	m_bIsConnected = FALSE;
	if(m_hDbc == NULL)
		return;
	SQLDisconnect(m_hDbc);
	SQLFree();
}

BOOL CLineageDatabase::Open(CHAR* szDSN,CHAR* szUser, CHAR* szPass)
{
	SQLRETURN ret;

	if(m_lConnectionTimeout > 0)
		SQLSetConnectAttr(m_hDbc, SQL_ATTR_CONNECTION_TIMEOUT, (SQLPOINTER)m_lConnectionTimeout, 0);

	SQLSetConnectAttr(m_hDbc, SQL_ATTR_LOGIN_TIMEOUT, (SQLPOINTER)m_lLoginTimeout, 0);

	ret = SQLConnectA(m_hDbc, 
		(SQLCHAR*)szDSN, 
		sizeof(szDSN), 
		(SQLCHAR*)szUser, 
		sizeof(szUser), 
		(SQLCHAR*)szPass, 
		sizeof(szPass));

	m_bIsConnected = ret == SQL_SUCCESS || ret == SQL_SUCCESS_WITH_INFO;
	return m_bIsConnected;
}

BOOL CLineageDatabase::Browse(UCHAR* szConnStrIn, UCHAR* szConnStrOut)
{
	SQLRETURN ret;
	SWORD swLenOut = 0;

	ret = SQLBrowseConnectA(m_hDbc, 
		(SQLCHAR*)szConnStrIn, 
		sizeof(szConnStrIn), 
		(SQLCHAR*)szConnStrOut, 
		MAX_BUFFER, 
		&swLenOut);

	m_bIsConnected = ret == SQL_SUCCESS || ret == SQL_SUCCESS_WITH_INFO;
	return m_bIsConnected;
}

void CLineageDatabase::SetConnectionTimeout(LONG nSeconds)
{
	if(m_hDbc)
		SQLSetConnectAttr(m_hDbc, SQL_ATTR_CONNECTION_TIMEOUT, (SQLPOINTER)nSeconds, 0);

	m_lConnectionTimeout = nSeconds;
}

BOOL CLineageDatabase::DriverConnect(CHAR* szConnStr, CHAR* szConnStrOut, HWND hWnd, enum drvCompletion drvConn)
{

	SQLRETURN ret;
	SQLSMALLINT pcbConnStrOut;

	if(drvConn == sqlPrompt && hWnd == NULL)
		return FALSE;

	if(m_lConnectionTimeout > 0)
		SQLSetConnectAttr(m_hDbc, SQL_ATTR_CONNECTION_TIMEOUT, (SQLPOINTER)m_lConnectionTimeout, 0);

	SQLSetConnectAttr(m_hDbc, SQL_ATTR_LOGIN_TIMEOUT, (SQLPOINTER)m_lLoginTimeout, 0);

	ret = SQLDriverConnectA(m_hDbc, 
		hWnd, 
		(SQLCHAR*)szConnStr, 
		SQL_NTS, 
		(SQLCHAR*)szConnStrOut,
		sizeof(szConnStrOut), 
		&pcbConnStrOut, 
		(SQLUSMALLINT)drvConn);

	m_bIsConnected = ret == SQL_SUCCESS || ret == SQL_SUCCESS_WITH_INFO;
	return m_bIsConnected;
}

void CLineageDatabase::SetReadOnly(BOOL bReadOnly)
{

	SQLSetConnectAttr(m_hDbc, SQL_ATTR_ACCESS_MODE, (SQLPOINTER)(bReadOnly? SQL_MODE_READ_ONLY : SQL_MODE_READ_WRITE), 0);
}

LONG CLineageDatabase::GetConnectionTimeout()
{
	LONG nSeconds;

	SQLGetConnectAttr(m_hDbc, SQL_ATTR_CONNECTION_TIMEOUT, &nSeconds, NULL, 0);

	return nSeconds;
}

/*BOOL CLineageDatabase::ExecuteA(CHAR *szSqlStr)
{
SQLRETURN ret;
SQLHSTMT hStmt = NULL;
SQLINTEGER nRowCount;

SQLAllocHandle(SQL_HANDLE_STMT, m_hDbc, &hStmt);
ret = SQLExecDirectA(hStmt, (SQLCHAR*)szSqlStr, SQL_NTS);

SQLRowCount(hStmt, &nRowCount);

m_nRowsAffected = nRowCount;
return ret == SQL_SUCCESS || ret == SQL_SUCCESS_WITH_INFO;
}

BOOL CLineageDatabase::ExecuteW(WCHAR *szSqlStr)
{
SQLRETURN ret;
SQLHSTMT hStmt = NULL;
SQLINTEGER nRowCount;

SQLAllocHandle(SQL_HANDLE_STMT, m_hDbc, &hStmt);
ret = SQLExecDirectW(hStmt, (SQLWCHAR*)szSqlStr, SQL_NTS);


SQLRowCount(hStmt, &nRowCount);

m_nRowsAffected = nRowCount;
return ret == SQL_SUCCESS || ret == SQL_SUCCESS_WITH_INFO;
}*/



/////////////////////////////////////////////////////////////////


void CLineageRecordset::AllocStmt()
{
	SQLAllocHandle(SQL_HANDLE_STMT, m_hDbc, &m_hStmt);
}

BOOL CLineageRecordset::OpenA(CHAR *szSqlStr)
{
	SQLRETURN ret;

	Close();
	m_hStmt = NULL;
	m_bIsEOF = FALSE;
	m_bIsBOF = FALSE;
	AllocStmt();

	ret = SQLExecDirectA(m_hStmt, (SQLCHAR*)szSqlStr, SQL_NTS);
	if(ret == SQL_SUCCESS || ret == SQL_SUCCESS_WITH_INFO)
	{
		SQLFetch(m_hStmt);
		return TRUE;
	}

	return FALSE;
}

BOOL CLineageRecordset::OpenW(WCHAR *szSqlStr)
{
	SQLRETURN ret;

	Close();
	m_hStmt = NULL;
	m_bIsEOF = FALSE;
	m_bIsBOF = FALSE;
	AllocStmt();

	ret = SQLExecDirectW(m_hStmt, (SQLWCHAR*)szSqlStr, SQL_NTS);
	if(ret == SQL_SUCCESS || ret == SQL_SUCCESS_WITH_INFO)
	{
		SQLFetch(m_hStmt);
		return TRUE;
	}

	return FALSE;
}

BOOL CLineageRecordset::GetFieldValue(int nField, CHAR *szData)
{
	SQLRETURN ret;
	SQLINTEGER cbValue;
	int nLength = GetFieldLength(nField) + 1;

	ret = SQLGetData(m_hStmt, (SQLUSMALLINT)nField + 1, SQL_C_CHAR, szData, nLength, &cbValue) == SQL_SUCCESS;
	return ret == SQL_SUCCESS || ret == SQL_SUCCESS_WITH_INFO;
}

BOOL CLineageRecordset::GetFieldValue(int nField, WCHAR *szData)
{
	SQLRETURN ret;
	SQLINTEGER cbValue;
	int nLength = GetFieldLength(nField) + 1;

	ret = SQLGetData(m_hStmt, (SQLUSMALLINT)nField + 1, SQL_WCHAR, szData, nLength, &cbValue) == SQL_SUCCESS;
	return ret == SQL_SUCCESS || ret == SQL_SUCCESS_WITH_INFO;
}

BOOL CLineageRecordset::GetFieldValue(int nField, LONG *lData)
{
	SQLRETURN ret;
	SQLINTEGER cbValue;
	int nLength = GetFieldLength(nField) + 1;

	ret = SQLGetData(m_hStmt, (SQLUSMALLINT)nField + 1, SQL_C_LONG, lData, nLength, &cbValue) == SQL_SUCCESS;
	return ret == SQL_SUCCESS || ret == SQL_SUCCESS_WITH_INFO;
}

BOOL CLineageRecordset::GetFieldValue(int nField, unsigned char *binData)
{
	SQLRETURN ret;
	SQLINTEGER cbValue;
	int nLength = GetFieldLength(nField) + 1;
	

	ret = SQLGetData(m_hStmt, (SQLUSMALLINT)nField + 1, SQL_C_BINARY, binData, nLength, &cbValue) == SQL_SUCCESS;
	return ret == SQL_SUCCESS || ret == SQL_SUCCESS_WITH_INFO;
}


BOOL CLineageRecordset::GetFieldValue(int nField, DOUBLE *dblData)
{

	SQLINTEGER cbValue;
	SQLRETURN ret;
	int nLength = GetFieldLength(nField) + 1;

	ret = SQLGetData(m_hStmt, (SQLUSMALLINT)nField + 1, SQL_C_DOUBLE, dblData, nLength, &cbValue) == SQL_SUCCESS;
	return ret == SQL_SUCCESS || ret == SQL_SUCCESS_WITH_INFO;
}

BOOL CLineageRecordset::GetFieldValue(int nField, struct tm* time)
{

	SQLINTEGER cbValue;
	SQLRETURN ret;
	int nLength = GetFieldLength(nField) + 1;
	SQL_TIMESTAMP_STRUCT* sqltm = new SQL_TIMESTAMP_STRUCT;

	ret = SQLGetData(m_hStmt, (SQLUSMALLINT)nField + 1, SQL_C_TYPE_TIMESTAMP, sqltm, nLength, &cbValue) == SQL_SUCCESS;
	if(ret == SQL_SUCCESS || ret == SQL_SUCCESS_WITH_INFO)
	{       
		time->tm_year = sqltm->year;
		time->tm_mon = sqltm->month - 1; //January must be = 0          
		time->tm_mday = sqltm->day;
		time->tm_hour = sqltm->hour;
		time->tm_min = sqltm->minute;
		time->tm_sec = sqltm->second;
		return TRUE;
	}
	return FALSE;
}


BOOL CLineageRecordset::MoveFirst()
{
	SQLRETURN ret;

	ret = SQLFetchScroll(m_hStmt, SQL_FETCH_FIRST, 0);

	m_bIsBOF = ret == SQL_SUCCESS || ret == SQL_SUCCESS_WITH_INFO;
	if(m_bIsBOF)
		m_bIsEOF = FALSE;

	return m_bIsBOF;
}

BOOL CLineageRecordset::MoveNext()
{
	SQLRETURN ret;

	ret = SQLFetchScroll(m_hStmt, SQL_FETCH_NEXT, 0);

	m_bIsEOF = ret == SQL_NO_DATA;
	m_bIsBOF = FALSE;
	return ret == SQL_SUCCESS || ret == SQL_SUCCESS_WITH_INFO;
}

BOOL CLineageRecordset::MovePrevious()
{
	SQLRETURN ret;

	ret = SQLFetchScroll(m_hStmt, SQL_FETCH_PRIOR, 0);
	m_bIsBOF = ret == SQL_NO_DATA;
	m_bIsEOF = FALSE;
	return ret == SQL_SUCCESS || ret == SQL_SUCCESS_WITH_INFO;
}

BOOL CLineageRecordset::MoveLast()
{
	SQLRETURN ret;

	ret = SQLFetchScroll(m_hStmt, SQL_FETCH_LAST, 0);

	m_bIsEOF = ret == SQL_SUCCESS || ret == SQL_SUCCESS_WITH_INFO;
	if(m_bIsEOF)
		m_bIsBOF = FALSE;

	return m_bIsEOF;
}

LONG CLineageRecordset::GetFieldLength(int nField)
{
	SQLSMALLINT fSqlType, ibScale, fNullable;
	SQLUINTEGER cbColDef;

	SQLDescribeCol(m_hStmt, nField + 1, NULL, 0, 0, &fSqlType, &cbColDef, &ibScale, &fNullable);

	return cbColDef;        
}


void CLineageRecordset::Close()
{
	if(m_hStmt != NULL)
		SQLFreeHandle(SQL_HANDLE_STMT, m_hStmt);
	m_hStmt = NULL;
}


//CStringW Quote(CStringW s)
CStdString Quote(CStdString s)
{
	int i=0;
	i=s.GetLength()-1;
	while(i>=0)
	{
		if (s[i] == L'\'') s.Insert(i,L'\'');
		i--;
	}
	 s.Format(L"\'%ls\'",s);
	//CLog_AddW(CLOG,0,L"Quote=[%s]",s);
	return s;
}