
#if !defined(_CRT_SECURE_NO_WARNINGS)
#define _CRT_SECURE_NO_WARNINGS
#endif

#pragma once


#include "Globals.h"
#include "stdstring.h"

#include <windows.h>
#include <sql.h>
#include <sqlext.h>
#include <time.h>

class CLineageDatabase
{
protected:
	LONG m_lLoginTimeout;
	LONG m_lConnectionTimeout;
	BOOL m_bIsConnected;
	SQLHENV m_hEnv;
	int m_nRowsAffected;

	void SQLFree();
	void SQLAlloc();

public:
	SQLHDBC m_hDbc;

	enum drvCompletion
	{
		sqlNoPrompt = SQL_DRIVER_NOPROMPT,
		sqlPrompt = SQL_DRIVER_PROMPT
	};

	CLineageDatabase()
	{
		m_hDbc = NULL;
		m_hEnv = NULL;
		m_lConnectionTimeout = 0;
		m_lLoginTimeout = 5;
		m_bIsConnected = FALSE;
		m_nRowsAffected = 0;
		SQLAlloc();
	};

	~CLineageDatabase()
	{
		Close();
		m_lConnectionTimeout = 0;
		m_lLoginTimeout = 0;
		m_bIsConnected = FALSE;
		m_nRowsAffected = 0;
	};

	BOOL Open(CHAR* lpstrDSN, CHAR* lpstrUser = NULL, CHAR* lpstrPass = NULL);
	BOOL DriverConnect(CHAR* szConnStr, CHAR* szConnStrOut = NULL, HWND hWnd = NULL, enum drvCompletion drvConn = sqlNoPrompt);
	void SetReadOnly(BOOL bReadOnly = TRUE);
	void SetConnectionTimeout(LONG nSeconds);
	LONG GetConnectionTimeout();
	void SetLoginTimeout(LONG nSeconds)
	{
		m_lLoginTimeout = nSeconds;
	};
/*	BOOL ExecuteA(CHAR* szSqlStr);
	BOOL ExecuteW(WCHAR* szSqlStr);
	int GetRowsAffected()
	{
		return m_nRowsAffected;
	};*/

	BOOL Browse(UCHAR* lpstrConnStrIn, UCHAR* lpstrConnStrOut);
	BOOL IsConnected()
	{
		return m_bIsConnected;
	};
	void Close();
};


class CLineageRecordset
{
public:
        enum datatypeEnum
        {
                typeChar = SQL_CHAR, 
                typeVarChar = SQL_VARCHAR, 
                typeLongVarChar = SQL_LONGVARCHAR, 
                typeWChar = SQL_WCHAR, 
                typeWVarChar = SQL_WVARCHAR,
                typeWLongVarChar = SQL_WLONGVARCHAR,
                typeDecimal = SQL_DECIMAL,
                typeNumeric = SQL_NUMERIC,
                typeSmallint = SQL_SMALLINT,
                typeInteger = SQL_INTEGER,
                typeReal = SQL_REAL,
                typeFloat = SQL_FLOAT,
                typeDouble = SQL_DOUBLE,
                typeBit = SQL_BIT,
                typeTinyint = SQL_TINYINT,
                typeBigInt = SQL_BIGINT,
                typeBinary = SQL_BINARY,
                typeVarBinary = SQL_VARBINARY,
                typeLongVarBinary =  SQL_LONGVARBINARY,
                typeDate = SQL_TYPE_DATE,
                typeTime = SQL_TYPE_TIME,
                typeTimeStamp = SQL_TYPE_TIMESTAMP,
                typeIntervalMonth = SQL_INTERVAL_MONTH,
                typeIntervalYear = SQL_INTERVAL_YEAR,
                typeIntervalYearToMonth = SQL_INTERVAL_YEAR_TO_MONTH,
                typeIntervalDay = SQL_INTERVAL_DAY,
                typeIntervalHour = SQL_INTERVAL_HOUR,
                typeIntervalMinute = SQL_INTERVAL_MINUTE,
                typeIntervalSecond = SQL_INTERVAL_SECOND,
                typeIntervalDayToHour = SQL_INTERVAL_DAY_TO_HOUR,
                typeIntervalDayToMinute = SQL_INTERVAL_DAY_TO_MINUTE,
                typeIntervalDayToSecond = SQL_INTERVAL_DAY_TO_SECOND,
                typeIntervalHourToMinute = SQL_INTERVAL_HOUR_TO_MINUTE,
                typeIntervalHourToSecond = SQL_INTERVAL_HOUR_TO_SECOND,
                typeIntervalMinuteToSecond = SQL_INTERVAL_MINUTE_TO_SECOND,
                typeGUID = SQL_GUID             
        };

        CLineageRecordset(CLineageDatabase* pDb)
        {
                m_hDbc = pDb->m_hDbc;
                m_hStmt = NULL;
                m_bIsEOF = FALSE;
                m_bIsBOF = FALSE;
				//AllocStmt();
		};

        ~CLineageRecordset()
        {
                Close();
                m_hStmt = NULL;
                m_bIsEOF = FALSE;
                m_bIsBOF = FALSE;
        };

        BOOL OpenA(CHAR* szSqlStr);
		BOOL OpenW(WCHAR* szSqlStr);

		LONG GetFieldLength(int nField);
        BOOL GetFieldValue(int nField, CHAR* szData);
		BOOL GetFieldValue(int nField, WCHAR *szData);
        BOOL GetFieldValue(int nField, LONG *lData);
		 BOOL GetFieldValue(int nField, unsigned char *binData);
        BOOL GetFieldValue(int nField, DOUBLE *dblData);
        BOOL GetFieldValue(int nField, struct tm* time);
        
        BOOL MoveFirst();
        BOOL MoveNext();
        BOOL MovePrevious();
        BOOL MoveLast();
        BOOL IsEof() 
                {return m_bIsEOF;};
        BOOL IsBof()
                {return m_bIsBOF;};
        
        void Close();

		CStdString GetError()
			
		{
			SQLWCHAR      SqlState[6];
			SQLWCHAR      ErrorMsg[SQL_MAX_MESSAGE_LENGTH];
			SQLINTEGER   NativeError;
			SQLSMALLINT  MsgLen;
			SQLRETURN diag = SQLGetDiagRec(SQL_HANDLE_STMT,m_hStmt,1,SqlState,
				&NativeError,ErrorMsg,sizeof(ErrorMsg), &MsgLen);
			//CStringW s;
			CStdString s;
			s.Format(L"SQL ERROR: [%s]",ErrorMsg);
			ErrorMsg;
			return s;
		};

		/*LONG RowCount()
		{
			SQLINTEGER nRowCount;
			SQLRowCount(m_hStmt, &nRowCount);
			return nRowCount;
		}*/

protected:
        BOOL m_bIsEOF;
        BOOL m_bIsBOF;
        SQLHDBC m_hDbc;
        SQLHSTMT m_hStmt;
        void AllocStmt();
};

extern CLineageDatabase g_DB;
extern CRITICAL_SECTION g_DBCS;

inline void DBCS_INIT()
{
	InitializeCriticalSection(&g_DBCS);
}

inline void DBCS_LOCK()
{
	EnterCriticalSection(&g_DBCS);
};

inline void DBCS_UNLOCK()
{
	LeaveCriticalSection(&g_DBCS);
};

inline void DBCS_DONE()
{
	DeleteCriticalSection(&g_DBCS);
};

CStdString Quote(CStdString s);
