////////////////////////////////////////////////////////////////////////////////
// 
// 2D Line Segment Intersection example
// Implementation of the theory provided by Paul Bourke
// 
// Written by Damian Coventry
// Tuesday, 9 January 2007
// 
////////////////////////////////////////////////////////////////////////////////

#include <iostream>


class Vector
{
public:
    float x_, y_;

    Vector(float f = 0.0f)
        : x_(f), y_(f) {}

    Vector(float x, float y)
        : x_(x), y_(y) {}
};

class LineSegment
{
public:
    Vector begin_;
    Vector end_;

    LineSegment(const Vector& begin, const Vector& end)
        : begin_(begin), end_(end) {}

    enum IntersectResult { PARALLEL, COINCIDENT, NOT_INTERESECTING, INTERESECTING };

    IntersectResult Intersect(const LineSegment& other_line, Vector& intersection)
    {
        float denom = ((other_line.end_.y_ - other_line.begin_.y_)*(end_.x_ - begin_.x_)) -
                      ((other_line.end_.x_ - other_line.begin_.x_)*(end_.y_ - begin_.y_));

        float nume_a = ((other_line.end_.x_ - other_line.begin_.x_)*(begin_.y_ - other_line.begin_.y_)) -
                       ((other_line.end_.y_ - other_line.begin_.y_)*(begin_.x_ - other_line.begin_.x_));

        float nume_b = ((end_.x_ - begin_.x_)*(begin_.y_ - other_line.begin_.y_)) -
                       ((end_.y_ - begin_.y_)*(begin_.x_ - other_line.begin_.x_));

        if(denom == 0.0f)
        {
            if(nume_a == 0.0f && nume_b == 0.0f)
            {
                return COINCIDENT;
            }
            return PARALLEL;
        }

        float ua = nume_a / denom;
        float ub = nume_b / denom;

        if(ua >= 0.0f && ua <= 1.0f && ub >= 0.0f && ub <= 1.0f)
        {
            // Get the intersection point.
            intersection.x_ = begin_.x_ + ua*(end_.x_ - begin_.x_);
            intersection.y_ = begin_.y_ + ua*(end_.y_ - begin_.y_);

            return INTERESECTING;
        }

        return NOT_INTERESECTING;
    }
};

int DoLineSegmentIntersection(const Vector& p0, const Vector& p1, const Vector& p2, const Vector& p3);
int DoLineSegmentIntersection_F(float p1x1, float p1y1, float p2x1, float p2y1, float p3x1, float p3y1, float p4x1, float p4y1);

struct FVectorInt
{
	int x;
	int y;
	int z_min;
	int z_max;
};

struct NoFlyZone
{
	int id; //zone ID
	__int8 pointCount;
	FVectorInt points[10];
};

struct MagicBarrierInfo
{
	int id; //barrier ID
	int x1; //point 1
	int y1;
	int z1;
	int x2; //point 2
	int y2;
	int z2;
	__int8 isZMatters; //0 not matters, 1, matters, 2 not matters for wyverns
	__int8 isForMoving; //0 not for moving, 1 for moving
	__int8 isForAttack; //0 not for attack, 1 for all attacks, 2 for physical attacks, 3 for magical attacks
	int htmMessageId; //see L2Server.cpp: sendMyHTML function

};

#define MagicBarrier_ARRAY_SIZE 100
#define NOFLY_ARRAY_SIZE 100
static MagicBarrierInfo BarrierArray[MagicBarrier_ARRAY_SIZE];
static NoFlyZone NoFlyArray[NOFLY_ARRAY_SIZE];
void IniBarrier();
void IniNoFlyZones();
bool wyvern_restrictor(int x, int y);
bool loot_restrictor(int x, int y);


int isBarrierLinesCossing(float x1, float y1, float x2, float y2, int z1, int action);
int is2LinesCossing(float l1x1, float l1y1, float l1x2, float l1y2, float l2x1, float l2y1, float l2x2, float l2y2, int z1, int action);

/*
collisions
       -     
    4   :

 #define WORLDCOLLISION_HASH_SIZE 100000
 CMwmrQueue<WorldCollisionElement,WORLDCOLLISION_HA SH_SIZE> WorldCollisionHash;

 ..    (    ),     push/pop   .    
  CMwmrQueue    ,    thread-safe    (   )




 -  plaincollision
     4   
           
    -     
   -      ,   
      
  :     -   





bool CGeoData::IsTargetVisible(FVector &vPosActor, FVector &vPosTarget, int nInZoneObjectId, bool bCheckPlaneCollision)
{
	bool bIsVisible = false;
	bIsVisible = this->_IsTargetVisible(vPosActor,vPosTarget);
	if (bIsVisible && bCheckPlaneCollision)
	{
		FVector vPosCollisionOut;
		//521920 CWorldPlaneCollision::CheckCollision(const FVector &vStart, const FVector &vEnd, double dRadius, FVector &Out>
		if (g_WorldPlaneCollision.CheckCollision(vPosActor,vPosTarget,1,vPosCollisionOut,nInZoneObjectId))
		{
			bIsVisible = false;
		}
	}
	return bIsVisible;
}

.text:004A066C                 mov     ecx, ebx
.text:004A066E                 call    dword ptr [edx+38h] //User_VirtualFunction
.text:004A0671                 mov     ecx, 18ACD030h
.text:004A0676                 call    CGeodata_GetGeodata_416C40
.text:004A067B                 mov     ecx, eax
.text:004A067D                 call    CGeoData___IsTargetVisible_vPosAtor__vPosTarget__47DDE0 //, int nInZoneObjectId, bool bCheckPlaneCollision)
.text:004A0682                 test    al, al
.text:004A0684                 jnz     short loc_4A06CB
.text:004A0686                 mov     edx, [ebx]
.text:004A0688                 mov     ecx, ebx
.text:004A068A                 call    dword ptr [edx+18h] ] //User_VirtualFunction
.text:004A068D                 test    al, al
.text:004A068F                 jz      short loc_4A06AE
*/
