#define _CRT_SECURE_NO_WARNINGS 1

#include "String.h"


String::String()
: m_CStr( new char[ 1 ] ) {
	m_CStr[ 0 ] = '\0';
}

String::String( const char *Str ) {
	m_CStr = new char[ strlen( Str ) + 1 ];
	strcpy( m_CStr, Str );
}

String::String( const std::string &Str ) {
	m_CStr = new char[ Str.length() + 1 ];
	strcpy( m_CStr, Str.c_str() );
}

String::String( const String &Str ) {
	m_CStr = Str.GetCStrCopy();
}

String::~String() {
	delete[] m_CStr;
}

void String::Set( const char *Str ) {
	char *NewStr = new char[ strlen( Str ) + 1 ];
	strcpy( NewStr, Str );
	delete[] m_CStr;
	m_CStr = NewStr;
}

void String::Set( const std::string &Str ) {
	Set( Str.c_str() );
}

void String::Set( const String &Str ) {
	delete[] m_CStr;
	m_CStr = Str.GetCStrCopy();
}

void String::Append( const char *Str ) {
	size_t NewLen = strlen( m_CStr ) + strlen( Str );
	char *NewStr = new char[ NewLen + 1 ];
	strcpy( NewStr, m_CStr );
	strcat( NewStr, Str );
	delete[] m_CStr;
	m_CStr = NewStr;
}

void String::Append( const std::string &Str ) {
	Append( Str.c_str() );
}

void String::Append( const String &Str ) {
	Append( Str.GetCStr() );
}

void String::Append( const int Num ) {
	char NumStr[11];
	_itoa( Num, NumStr, 10 );
	Append( NumStr );
}

bool String::Replace( const char Find, const char Replacer ) {
	for ( size_t i = 0;i < strlen( m_CStr );++ i ) {
		if ( m_CStr[ i ] == Find ) {
			m_CStr[ i ] = Replacer;
			return true;
		}
	}

	return false;
}

void String::ReplaceAll( const char Find, const char Replacer ) {
	/* 
	 * We could loop with String::Replace() until false is returned,
	 * but this would result in big performance lose on much Find occurences
	 */
	for ( size_t i = 0;i < strlen( m_CStr );++ i ) {
		if ( m_CStr[ i ] == Find ) {
			m_CStr[ i ] = Replacer;
		}
	}
}

size_t String::Trim( const char Ch ) {
	size_t CurLen = strlen( m_CStr );
	size_t j = 0;

	for ( size_t i = 0;i < CurLen;++ i ) {
		if ( m_CStr[ i ] == Ch ) {
			for ( j = i;j < CurLen;++ j ) {
				m_CStr[ j ] = m_CStr[ j + 1 ];
			}
		}
	}

	return CurLen - strlen( m_CStr ); 
}

void String::Reverse()
{
	size_t CurLen = strlen( m_CStr );
	size_t BackPos = 0;
	char Ch = 0;

	for ( size_t i = 0;i < CurLen;++ i ) {
		BackPos = CurLen - ( i + 1 );

		if ( BackPos <= i ) {
			break;
		}

		Ch = m_CStr[ i ];
		m_CStr[ i ] = m_CStr[ BackPos ];
		m_CStr[ BackPos ] = Ch;
	}
}

std::vector<String> &String::GetSplit( const char Ch ) const {
	size_t CurLen = strlen( m_CStr );
	size_t Start = 0;
	std::vector<String> *StrSplits = new std::vector<String>;

	for ( size_t i = 0;i < CurLen;++ i ) {
		if ( m_CStr[ i ] == Ch ) {
			StrSplits->push_back( GetSubstr( Start, i - Start ) );
			Start = i + 1;
		}
	}

	StrSplits->push_back( GetSubstr( Start, CurLen - Start ) );
	return *StrSplits;
}

String &String::GetSubstr( size_t Start, size_t Len ) const {
	char *NewCStr = new char[ Len + 1 ];
	String *NewStr = 0;
	strncpy( NewCStr, m_CStr + Start, Len );
	NewCStr[ Len ] = '\0';
	NewStr = new String( NewCStr );
	delete[] NewCStr;
	return *NewStr;
}

bool String::IsStartingWith( const char *Str ) const {
	size_t CurLen = strlen( m_CStr );

	for ( size_t i = 0;i < strlen( Str );++ i) {
		if ( i > CurLen - 1 ) {
			return false;
		} else if ( Str[ i ] != m_CStr[ i ] ) {
			return false;
		}
	}

	return true;
}

bool String::IsEquals( const char *Str ) const {
	size_t CurLen = strlen( m_CStr );
	size_t StrLen = strlen( Str );

	if ( CurLen != StrLen ) {
		return false;
	}

	for ( size_t i = 0;i < StrLen;++ i ) {
		if ( Str[ i ] != m_CStr[ i ] ) {
			return false;
		}
	}

	return true;
}

const char *String::GetCStr() const {
	return m_CStr;
}

char *String::GetCStrCopy() const {
	char *NewCStr = new char[ strlen( m_CStr ) + 1 ];
	strcpy( NewCStr, m_CStr );
	return NewCStr;
}

size_t String::GetLen() const {
	return strlen( m_CStr );
}
