#ifndef __CSTM_STRING_H__
#define __CSTM_STRING_H__

#include <string>
#include <vector>

class String {
public:
	/* constructor, sets m_CStr to an empty string */
	String();
	/* constructor, sets m_CStr to a copy of Str */
	String( const char *Str );
	/* constructor, sets m_CStr to a copy of Str.c_str() */
	String( const std::string &Str );
	/* constructor, sets m_CStr to Str.GetCStrCopy() */
	String( const String &Str );
	/* destructor, cleans m_CStr */
	virtual ~String();

	/* sets m_CStr to a copy of Str */
	void Set( const char *Str );
	/* sets m_CStr to a copy of Str.c_str() */
	void Set( const std::string &Str );
	/* sets m_CStr to Str.GetCStrCopy() */
	void Set( const String &Str );

	/* appends Str to m_CStr*/
	void Append( const char *Str );
	/* appends Str.c_str() to m_CStr */
	void Append( const std::string &Str );
	/* appends Str.GetCStr() to m_CStr */
	void Append( const String &Str );
	/* appends Num as C Sstyle string to m_CStr */
	void Append( const int Num );

	/* replaces the first occurence of Find with Replacer */
	bool Replace( const char Find, const char Replacer );
	/* replaces all occurences of Find with Replacer */
	void ReplaceAll( const char Find, const char Replacer );

	/* Removes all ocurences of Ch from m_CStr */
	size_t Trim(const char Ch);

	/* reverses the string */
	void Reverse();

	/* splits the string by Ch and returns a list of splits */
	std::vector<String> &GetSplit( const char Ch ) const;

	/* returns a substring, starting at Start with length of Len */
	String &GetSubstr( size_t Start, size_t Len ) const;

	/* returns true if m_CStr starts with Str */
	bool IsStartingWith( const char *Str ) const;
	/* returns true if m_CStr is equals Str */
	bool IsEquals( const char *Str ) const;

	/* returns m_CStr as const char* */
	const char *GetCStr() const;
	/* returns a copy of m_CStr */
	char *GetCStrCopy() const;
	/* returns the size of the string */
	size_t GetLen() const;
private:
	char *m_CStr;
};

#endif